@@ -32,39 +32,32 @@
 
 #include "config.h"
 
-#include <string>
 #include <algorithm>
-#include <sstream>
-#include <vector>
-#include <utility>
-#include <mutex>
-#include <thread>
 #include <array>
+#include <mutex>
+#include <sstream>
+#include <string>
+#include <thread>
+#include <utility>
+#include <vector>
 
 #include <curl/curl.h>
 
-#include "upnpapi.h"
-#include "httputils.h"
-#include "ssdplib.h"
-#include "soaplib.h"
-#include "ThreadPool.h"
-#include "upnp_timeout.h"
 #include "genut.h"
-#include "netif.h"
+#include "httputils.h"
+#include "soaplib.h"
+#include "upnpapi.h"
+#include "uri.h"
 
 /* Needed for GENA */
 #include "gena.h"
-#include "gena_ctrlpt.h"
 #include "gena_device.h"
 #include "gena_sids.h"
-#include "miniserver.h"
-#include "service_table.h"
 
 #ifdef INTERNAL_WEB_SERVER
-#include "VirtualDir.h"
 #include "webserver.h"
-#include <sys/stat.h>
 #include <fcntl.h>
+#include <sys/stat.h>
 #ifndef _WIN32
 # ifndef O_BINARY
 #  define O_BINARY 0
@@ -73,15 +66,10 @@
 
 #endif /* INTERNAL_WEB_SERVER */
 
-#include <cassert>
-#include <csignal>
 #include <cstdlib>
-#include <cstring>
 
 #ifndef _WIN32
 #include <unistd.h>
-#include <sys/param.h>
-#include <sys/types.h>
 #else
 #include <io.h>
 #endif
@@ -129,6 +117,9 @@ bool g_use_all_interfaces;
 
 /* General option flags */
 unsigned int g_optionFlags;
+/* SSDP bootid and configid. These default to 1, but should be managed by our user */
+int g_bootidUpnpOrg{1};
+int g_configidUpnpOrg{1};
 
 /* Local global options, usually set from the options list of initWithOptions */
 static int o_networkWaitSeconds = 60;
@@ -143,7 +134,7 @@ unsigned short LOCAL_PORT_V6;
 
 /*! UPnP device and control point handle table    */
 #define NUM_HANDLE 200
-static std::array<Handle_Info*, NUM_HANDLE> HandleTable;
+static std::array<struct Handle_Info*, NUM_HANDLE> HandleTable;
 
 /*! Maximum content-length (in bytes) that the SDK will process on an incoming
  * packet. Content-Length exceeding this size will be not processed and
@@ -314,13 +305,11 @@ static int getIfInfo(const char *IfNames)
     if (!using_ipv6()) {
         // Trim the ipv6 addresses
         for (auto& netif : g_netifs) {
-            auto addrmasks = netif.getaddresses();
+            auto [addrs, _] = netif.getaddresses();
             std::vector<NetIF::IPAddr> kept;
-            std::copy_if(addrmasks.first.begin(), addrmasks.first.end(),
-                         std::back_inserter(kept),
-                         [](const NetIF::IPAddr &addr){
-                             return addr.family() ==
-                                 NetIF::IPAddr::Family::IPV4;});
+            for (const auto& addr : addrs)
+                if (addr.family() == NetIF::IPAddr::Family::IPV4)
+                    kept.push_back(addr);
             netif.trimto(kept);
         }
     }
@@ -338,7 +327,7 @@ static int getIfInfo(const char *IfNames)
 static int getmyipv4(const char *inipv4 = nullptr)
 {
     bool ipspecified = (nullptr != inipv4 && 0 != inipv4[0]);
-    NetIF::Interfaces *ifs = NetIF::Interfaces::theInterfaces();
+    const NetIF::Interfaces *ifs = NetIF::Interfaces::theInterfaces();
     NetIF::Interface *netifp{nullptr};
     NetIF::Interfaces::Filter filt;
     filt.needs = {NetIF::Interface::Flags::HASIPV4,
@@ -355,8 +344,7 @@ static int getmyipv4(const char *inipv4 = nullptr)
 
     if (ipspecified) {
         for (auto& iface : selected) {
-            const auto addrs = iface.getaddresses().first;
-            for (const auto& addr : addrs) {
+            for (const auto& addr : iface.getaddresses().first) {
                 if (addr.straddr() == inipv4) {
                     netifp = &iface;
                     goto ipv4found;
@@ -422,7 +410,6 @@ static int WinsockInit(void)
 /* Initializes the global threadm pools used by the UPnP SDK. */
 static int UpnpInitThreadPools()
 {
-    int ret = UPNP_E_SUCCESS;
     ThreadPoolAttr attr;
 
     attr.maxThreads = MAX_THREADS;
@@ -432,18 +419,16 @@ static int UpnpInitThreadPools()
     attr.maxIdleTime = THREAD_IDLE_TIME;
     attr.maxJobsTotal = MAX_JOBS_TOTAL;
 
-    auto i = std::any_of(o_threadpools.begin(), o_threadpools.end(), [&](const std::pair<ThreadPool*, const char*> &entry)
-        { return entry.first->start(&attr) != UPNP_E_SUCCESS; });
-
-    if (i) {
-        ret = UPNP_E_INIT_FAILED;
-        UpnpSdkInit = 0;
-        UpnpFinish();
+    for (const auto& [tp, _] : o_threadpools) {
+        if (tp->start(&attr) != UPNP_E_SUCCESS) {
+            UpnpSdkInit = 0;
+            UpnpFinish();
+            return UPNP_E_INIT_FAILED;
+        }
     }
-    return ret;
+    return UPNP_E_SUCCESS;
 }
 
-
 /*!
  * \brief Performs the initial steps in initializing the UPnP SDK.
  *
@@ -539,8 +524,7 @@ static int UpnpInitStartServers(unsigned short DestPort)
     LOCAL_PORT_V4 = LOCAL_PORT_V6 = DestPort;
     retVal = StartMiniServer(&LOCAL_PORT_V4, &LOCAL_PORT_V6);
     if (retVal != UPNP_E_SUCCESS) {
-        UpnpPrintf(UPNP_CRITICAL, API, __FILE__, __LINE__,
-                   "Miniserver start error\n");
+        UpnpPrintf(UPNP_CRITICAL, API, __FILE__, __LINE__, "Miniserver start error\n");
         UpnpFinish();
         return retVal;
     }
@@ -549,8 +533,7 @@ static int UpnpInitStartServers(unsigned short DestPort)
 #if EXCLUDE_WEB_SERVER == 0
     retVal = UpnpEnableWebserver(WEB_SERVER_ENABLED);
     if (retVal != UPNP_E_SUCCESS) {
-        UpnpPrintf(UPNP_CRITICAL, API, __FILE__, __LINE__,
-                   "Webserver start error\n");
+        UpnpPrintf(UPNP_CRITICAL, API, __FILE__, __LINE__, "Webserver start error\n");
         UpnpFinish();
         return retVal;
     }
@@ -560,8 +543,7 @@ static int UpnpInitStartServers(unsigned short DestPort)
 }
 
 /* Only use HostIP if ifName is nullptr (meaning a call from UpnpInit()) */
-static int waitForNetwork(
-    const char *HostIP, const char *ifName)
+static int waitForNetwork(const char *HostIP, const char *ifName)
 {
     int ret = UPNP_E_INVALID_INTERFACE;
 
@@ -582,8 +564,7 @@ static int waitForNetwork(
 
         if (!NetIF::Interfaces::theInterfaces()->refresh()) {
             UpnpPrintf(UPNP_ERROR, API, __FILE__, __LINE__,
-                       "UpnpInit: could not read network interface state "
-                       "from system\n");
+                       "UpnpInit: could not read network interface state from system\n");
         }
         std::this_thread::sleep_for(std::chrono::seconds(loop_sleep));
     }
@@ -592,12 +573,11 @@ static int waitForNetwork(
 
 /* If ifName is nullptr this is a call from UpnpInit(). UpnpInit2() calls us with
  * a non-null, possibly empty string */
-static int upnpInitCommon(const char *hostIP, const char *ifName,
-                          unsigned short DestPort)
+static int upnpInitCommon(const char *hostIP, const char *ifName, unsigned short DestPort)
 {
     int retVal = UPNP_E_SUCCESS;
 
-    std::unique_lock<std::mutex> lck(gSDKInitMutex);
+    std::scoped_lock lck(gSDKInitMutex);
 
     /* Check if we're already initialized. */
     if (UpnpSdkInit == 1) {
@@ -610,9 +590,8 @@ static int upnpInitCommon(const char *hostIP, const char *ifName,
     if (retVal != UPNP_E_SUCCESS) {
         goto exit_function;
     }
-    UpnpPrintf(UPNP_INFO, API, __FILE__, __LINE__,
-               "UpnpInit: hostIP=%s, ifName=%s, DestPort=%d.\n",
-               hostIP ? hostIP : "",ifName?ifName:"",static_cast<int>(DestPort));
+    UpnpPrintf(UPNP_INFO, API, __FILE__, __LINE__, "UpnpInit: hostIP=%s, ifName=%s, DestPort=%d.\n",
+               hostIP ? hostIP : "",ifName?ifName:"", static_cast<int>(DestPort));
 
     retVal = waitForNetwork(hostIP, ifName);
 
@@ -639,8 +618,7 @@ static int upnpInitCommon(const char *hostIP, const char *ifName,
         UpnpSdkInit = 0;
         goto exit_function;
     }
-    UpnpPrintf(UPNP_INFO, API, __FILE__, __LINE__,
-               "Upnpinit: retVal %d Host Ip: %s Host Port: %d\n",
+    UpnpPrintf(UPNP_INFO, API, __FILE__, __LINE__, "Upnpinit: retVal %d Host Ip: %s Host Port: %d\n",
                retVal, g_netifs.begin()->firstipv4addr()->straddr().c_str(),
                static_cast<int>(LOCAL_PORT_V4));
 
@@ -655,8 +633,7 @@ EXPORT_SPEC int UpnpInit(const char *hostIP, unsigned short DestPort)
 
 EXPORT_SPEC int UpnpInit2(const char *IfName, unsigned short DestPort)
 {
-    return UpnpInitWithOptions(IfName, DestPort,
-                               UPNP_FLAG_IPV6, UPNP_OPTION_END);
+    return UpnpInitWithOptions(IfName, DestPort, UPNP_FLAG_IPV6, UPNP_OPTION_END);
 }
 
 EXPORT_SPEC int UpnpInit2(const std::vector<std::string>& ifnames, unsigned short port)
@@ -684,6 +661,20 @@ EXPORT_SPEC int UpnpInitWithOptions(
         switch (option) {
         case UPNP_OPTION_NETWORK_WAIT:
             o_networkWaitSeconds = va_arg(ap, int);
+            if (o_networkWaitSeconds <= 0)
+                o_networkWaitSeconds = 60;
+            break;
+        case UPNP_OPTION_BOOTID:
+            g_bootidUpnpOrg = va_arg(ap, int);
+            if (g_bootidUpnpOrg <= 0)
+                g_bootidUpnpOrg = 1;
+            break;
+        case UPNP_OPTION_NEXTBOOTID:
+            break;
+        case UPNP_OPTION_CONFIGID:
+            g_configidUpnpOrg = va_arg(ap, int);
+            if (g_configidUpnpOrg <= 0)
+                g_configidUpnpOrg = 1;
             break;
         default:
             UpnpPrintf(UPNP_CRITICAL, API, __FILE__, __LINE__,
@@ -751,8 +742,8 @@ void PrintThreadPoolStats(
                stats.totalIdleTime);
 }
 #else
-static UPNP_INLINE void PrintThreadPoolStats(
-    ThreadPool *, const char *, int, const char *)
+static void PrintThreadPoolStats(
+    ThreadPool*, const char*, int, const char*)
 {
 }
 #endif /* DEBUG */
@@ -792,15 +783,15 @@ EXPORT_SPEC int UpnpFinish()
 #if EXCLUDE_WEB_SERVER == 0
     web_server_destroy();
 #endif
-    for (const auto& entry : o_threadpools) {
-        entry.first->shutdown();
-        PrintThreadPoolStats(entry.first, __FILE__, __LINE__,
-                             entry.second);
+    for (const auto& [t, d] : o_threadpools) {
+        t->shutdown();
+        PrintThreadPoolStats(t, __FILE__, __LINE__, d);
     }
     /* remove all virtual dirs */
     UpnpRemoveAllVirtualDirs();
     UpnpSdkInit = 0;
     UpnpCloseLog();
+    NetIF::Interfaces::cleanup();
 
     return UPNP_E_SUCCESS;
 }
@@ -812,6 +803,7 @@ EXPORT_SPEC int UpnpSetWebRequestHostValidateCallback(
     g_hostvalidatecookie = cookie;
     return UPNP_E_SUCCESS;
 }
+
 EXPORT_SPEC int UpnpSetHostValidateCallback(
     WebCallback_HostValidate callback, void *cookie)
 {
@@ -823,8 +815,7 @@ EXPORT_SPEC std::string UpnpGetUrlHostPortForClient(const struct sockaddr_storag
 {
     NetIF::IPAddr claddr(reinterpret_cast<const struct sockaddr*>(clsock));
     NetIF::IPAddr hostaddr;
-    const NetIF::Interface *itf =
-        NetIF::Interfaces::interfaceForAddress(claddr, g_netifs, hostaddr);
+    const NetIF::Interface *itf = NetIF::Interfaces::interfaceForAddress(claddr, g_netifs, hostaddr);
     if (nullptr == itf) {
         return {};
     }
@@ -1009,11 +1000,9 @@ static int registerRootDeviceAllForms(
     }
 
     if (LowerDescUrl == nullptr)
-        upnp_strlcpy(HInfo->LowerDescURL, HInfo->DescURL,
-                     sizeof(HInfo->LowerDescURL));
+        upnp_strlcpy(HInfo->LowerDescURL, HInfo->DescURL, sizeof(HInfo->LowerDescURL));
     else
-        upnp_strlcpy(HInfo->LowerDescURL, LowerDescUrl,
-                     sizeof(HInfo->LowerDescURL));
+        upnp_strlcpy(HInfo->LowerDescURL, LowerDescUrl, sizeof(HInfo->LowerDescURL));
     UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
                "Root Device URL for legacy CPs: %s\n", HInfo->LowerDescURL);
     HInfo->HType = HND_DEVICE;
@@ -1024,21 +1013,18 @@ static int registerRootDeviceAllForms(
     HInfo->MaxSubscriptionTimeOut = UPNP_INFINITE;
 
     UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-               "registerRootDeviceAllForms: Ok Description at : %s\n",
-               HInfo->DescURL);
+               "registerRootDeviceAllForms: Ok Description at : %s\n", HInfo->DescURL);
 
 #if EXCLUDE_GENA == 0
     /*
      * GENA SET UP
      */
-    hasServiceTable = initServiceTable(HInfo->devdesc, &HInfo->ServiceTable);
+    hasServiceTable = initServiceTable(HInfo->devdesc, HInfo->serviceTable);
     if (hasServiceTable) {
-        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-                   "registerRootDeviceAllForms: GENA services:\n");
-        printServiceTable(&HInfo->ServiceTable, UPNP_ALL, API);
+        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,"registerRootDeviceAllForms: GENA services:\n");
+        printServiceTable(HInfo->serviceTable, UPNP_ALL, API);
     } else {
-        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-                   "\nUpnpRegisterRootDeviceAF: no services\n");
+        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__, "\nUpnpRegisterRootDeviceAF: no services\n");
     }
 #endif /* EXCLUDE_GENA */
 
@@ -1050,11 +1036,9 @@ exit_function:
 }
 
 EXPORT_SPEC int UpnpRegisterRootDevice(
-    const char *DescUrl, Upnp_FunPtr Fun, const void *Cookie,
-    UpnpDevice_Handle *Hnd)
+    const char *DescUrl, Upnp_FunPtr Fun, const void *Cookie, UpnpDevice_Handle *Hnd)
 {
-    return registerRootDeviceAllForms(
-        UPNPREG_URL_DESC, DescUrl, Fun, Cookie, Hnd, nullptr);
+    return registerRootDeviceAllForms( UPNPREG_URL_DESC, DescUrl, Fun, Cookie, Hnd, nullptr);
 }
 
 EXPORT_SPEC int UpnpRegisterRootDevice2(
@@ -1062,20 +1046,17 @@ EXPORT_SPEC int UpnpRegisterRootDevice2(
     size_t, int /*ignored*/, Upnp_FunPtr Fun, const void *Cookie,
     UpnpDevice_Handle *Hnd)
 {
-    return registerRootDeviceAllForms(
-        descriptionType, description_const, Fun, Cookie, Hnd, nullptr);
+    return registerRootDeviceAllForms(descriptionType, description_const, Fun, Cookie, Hnd, nullptr);
 }
 
 EXPORT_SPEC int UpnpRegisterRootDevice4(
     const char *DescUrl, Upnp_FunPtr Fun, const void *Cookie,
     UpnpDevice_Handle *Hnd, int /*AddressFamily*/, const char *LowerDescUrl)
 {
-    return registerRootDeviceAllForms(
-        UPNPREG_URL_DESC, DescUrl, Fun, Cookie, Hnd, LowerDescUrl);
+    return registerRootDeviceAllForms(UPNPREG_URL_DESC, DescUrl, Fun, Cookie, Hnd, LowerDescUrl);
 }
 
-EXPORT_SPEC int UpnpDeviceSetProduct(
-    UpnpDevice_Handle Hnd, const char *product, const char *version)
+EXPORT_SPEC int UpnpDeviceSetProduct(UpnpDevice_Handle Hnd, const char *product, const char *version)
 {
     struct Handle_Info *HInfo = nullptr;
     if (UpnpSdkInit != 1) {
@@ -1178,11 +1159,9 @@ int UpnpRegisterClient(Upnp_FunPtr Fun, const void *Cookie, UpnpClient_Handle *H
     return UPNP_E_SUCCESS;
 }
 
-void UpnpClientSetProduct(
-    UpnpClient_Handle, const char *product, const char *version)
+void UpnpClientSetProduct(UpnpClient_Handle, const char *product, const char *version)
 {
-    if (nullptr == product || 0 == *product ||
-        nullptr == version || 0 == *version) {
+    if (nullptr == product || 0 == *product || nullptr == version || 0 == *version) {
         return;
     }
     // We just have one client connection, so let httputils store the thing.
@@ -1298,8 +1277,7 @@ static int GetDescDocumentAndURL(
         retVal = UpnpDownloadUrlItem(globurl.c_str(), &descstr, nullptr);
         if (retVal != UPNP_E_SUCCESS) {
             UpnpPrintf(UPNP_ERROR, API, __FILE__, __LINE__,
-                       "UpnpRegisterRootDevice: error downloading doc: %d\n",
-                       retVal);
+                       "UpnpRegisterRootDevice: error downloading doc: %d\n", retVal);
             return retVal;
         }
         descdata = descstr;
@@ -1347,8 +1325,7 @@ static int GetDescDocumentAndURL(
     break;
     default:
         UpnpPrintf(UPNP_ERROR, API, __FILE__, __LINE__,
-                   "UpnpRegisterRootDevice: bad desc type %d\n",
-                   int(descriptionType));
+                   "UpnpRegisterRootDevice: bad desc type %d\n", int(descriptionType));
         return UPNP_E_INVALID_DESC;
     }
 
@@ -1358,8 +1335,7 @@ static int GetDescDocumentAndURL(
         upnp_strlcpy(descURL, localurl.c_str(), LINE_SIZE);
         desc = UPnPDeviceDesc(localurl, descdata);
         if (desc.ok && !simplename.empty()) {
-            web_server_set_localdoc(std::string("/") + simplename,
-                                    descdata, modtime);
+            web_server_set_localdoc(std::string("/") + simplename, descdata, modtime);
         }
     }
 
@@ -1398,8 +1374,7 @@ static int GetDescDocumentAndURL(
     retVal = UpnpDownloadUrlItem(description, &descstr, 0);
     if (retVal != UPNP_E_SUCCESS) {
         UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-                   "UpnpRegisterRootDevice: error downloading doc: %d\n",
-                   retVal);
+                   "UpnpRegisterRootDevice: error downloading doc: %d\n", retVal);
         return retVal;
     }
     desc = UPnPDeviceDesc(description, descstr);
@@ -1426,24 +1401,15 @@ int UpnpSendAdvertisement(UpnpDevice_Handle Hnd, int Exp)
     return UpnpSendAdvertisementLowPower(Hnd, Exp, -1, -1, -1);
 }
 
-struct upnp_timeout_data_int : public upnp_timeout_data {
-    int exp;
-};
-
 class AutoAdvertiseJobWorker : public JobWorker {
 public:
-    explicit AutoAdvertiseJobWorker(upnp_timeout *ev)
-        : m_event(ev) {}
-    ~AutoAdvertiseJobWorker() override {
-        deleteZ(m_event);
-    }
-    AutoAdvertiseJobWorker(const AutoAdvertiseJobWorker&) = delete;
-    AutoAdvertiseJobWorker& operator=(const AutoAdvertiseJobWorker&) = delete;
+    explicit AutoAdvertiseJobWorker(int h, int e)
+        : handle(h), exp(e) {}
     void work() override {
-        int exp = dynamic_cast<upnp_timeout_data_int*>(m_event->Event)->exp;
-        UpnpSendAdvertisement(m_event->handle, exp);
+        UpnpSendAdvertisement(handle, exp);
     }
-    upnp_timeout *m_event;
+    int handle;
+    int exp;
 };
 
 
@@ -1478,25 +1444,17 @@ int UpnpSendAdvertisementLowPower(
     if(retVal != UPNP_E_SUCCESS)
         return retVal;
 
-    auto adEvent = new upnp_timeout;
-    auto adEventData = new upnp_timeout_data_int;
-    adEventData->exp = Exp;
-    adEvent->Event = adEventData;
-    adEvent->handle = Hnd;
-
     if (checkLockHandle(HND_DEVICE, Hnd, &SInfo) == HND_INVALID) {
-        delete adEvent;
         return UPNP_E_INVALID_HANDLE;
     }
-
 #ifdef SSDP_PACKET_DISTRIBUTE
     time_t thetime = ((Exp / 2) - (AUTO_ADVERTISEMENT_TIME));
 #else
     time_t thetime = Exp - AUTO_ADVERTISEMENT_TIME;
 #endif
-    auto worker = std::make_unique<AutoAdvertiseJobWorker>(adEvent);
+    auto worker = std::make_unique<AutoAdvertiseJobWorker>(Hnd, Exp);
     retVal = gTimerThread->schedule(TimerThread::SHORT_TERM, TimerThread::REL_SEC, thetime,
-                                    &adEvent->eventId, std::move(worker));
+                                    nullptr, std::move(worker));
     HandleUnlock();
     return retVal;
 
@@ -1508,33 +1466,58 @@ int UpnpSendAdvertisementLowPower(
 #if EXCLUDE_SSDP == 0
 #ifdef INCLUDE_CLIENT_APIS
 
-
-int UpnpSearchAsync(
-    UpnpClient_Handle Hnd, int Mx, const char *Target, const void *Cookie)
+// Internal merged multicast / unicast search. This is unicast iff Mx is 0, in which case, saddress
+// and port must be set.
+static int searchAsyncUniMulti(
+    UpnpClient_Handle hnd, int mx, const char *target, const char *shost, int port, void *cookie)
 {
-    struct Handle_Info *SInfo = nullptr;
-    int retVal;
-
     if (UpnpSdkInit != 1) {
         return UPNP_E_FINISH;
     }
-    if (Target == nullptr) {
+    if (nullptr == target) {
         return UPNP_E_INVALID_PARAM;
     }
 
-    if (checkLockHandle(HND_CLIENT, Hnd, &SInfo, true) == HND_INVALID) {
+    struct Handle_Info *SInfo = nullptr;
+    if (checkLockHandle(HND_CLIENT, hnd, &SInfo, true) == HND_INVALID) {
         return UPNP_E_INVALID_HANDLE;
     }
-    if(Mx < 1)
-        Mx = DEFAULT_MX;
+    if (mx == 0) {
+        // unicast. Need address
+        if (nullptr == shost || shost[0] == 0) {
+            return UPNP_E_INVALID_PARAM;
+        }
+    } else {
+        // multicast. Don't want address
+        if (nullptr != shost && shost[0] != 0) {
+            return UPNP_E_INVALID_PARAM;
+        }
+    }
 
     HandleUnlock();
-    retVal = SearchByTarget(Mx, const_cast<char *>(Target), const_cast<void *>(Cookie));
-    if (retVal != 1)
-        return retVal;
-
-    return UPNP_E_SUCCESS;
+    return SearchByTarget(mx, target, shost, port, cookie);
 }
+
+int UpnpSearchAsync(UpnpClient_Handle hnd, int mx, const char *target, const void *cookie)
+{
+    if (mx < 1)
+        mx = DEFAULT_MX;
+    return searchAsyncUniMulti(hnd, mx, target, "", 0, const_cast<void *>(cookie));
+}
+
+int UpnpSearchAsyncUnicast(
+    UpnpClient_Handle hnd, const std::string& url, const char *target, void *cookie)
+{
+    uri_type parseduri;
+    int ret = parse_uri(url, &parseduri);
+    if (ret != UPNP_E_SUCCESS) {
+        return ret;
+    }
+    const char *host = parseduri.hostport.strhost.c_str();
+    int port = parseduri.hostport.strport.empty() ? 80 : atoi(parseduri.hostport.strport.c_str());
+    return searchAsyncUniMulti(hnd, 0, target, host, port, cookie);
+}
+
 #endif /* INCLUDE_CLIENT_APIS */
 #endif
 
@@ -1571,8 +1554,7 @@ int UpnpSetMaxSubscriptions(UpnpDevice_Handle Hnd, int MaxSubscriptions)
 
 
 #ifdef INCLUDE_DEVICE_APIS
-int UpnpSetMaxSubscriptionTimeOut(UpnpDevice_Handle Hnd,
-                                  int MaxSubscriptionTimeOut)
+int UpnpSetMaxSubscriptionTimeOut(UpnpDevice_Handle Hnd, int MaxSubscriptionTimeOut)
 {
     struct Handle_Info *SInfo = nullptr;
 
@@ -1597,8 +1579,35 @@ int UpnpSetMaxSubscriptionTimeOut(UpnpDevice_Handle Hnd,
 #endif /* INCLUDE_DEVICE_APIS */
 
 #ifdef INCLUDE_CLIENT_APIS
-int UpnpSubscribe(
-    UpnpClient_Handle Hnd, const char *EvtUrl, int *TimeOut, Upnp_SID& SubsId)
+int UpnpSubsOpsTimeoutMs(UpnpClient_Handle Hnd, int TimeOutMS)
+{
+    int retVal;
+    struct Handle_Info *SInfo = nullptr;
+
+    if (UpnpSdkInit != 1) {
+        retVal = UPNP_E_FINISH;
+        goto exit_function;
+    }
+
+    if (TimeOutMS <= 0) {
+        retVal = UPNP_E_INVALID_PARAM;
+        goto exit_function;
+    }
+
+    if (checkLockHandle(HND_CLIENT, Hnd, &SInfo, true) == HND_INVALID) {
+        retVal = UPNP_E_INVALID_HANDLE;
+        goto exit_function;
+    }
+    SInfo->SubsOpsTimeoutMS = TimeOutMS;
+    HandleUnlock();
+
+    retVal = UPNP_E_SUCCESS;
+
+exit_function:
+    return retVal;
+}
+
+int UpnpSubscribe(UpnpClient_Handle Hnd, const char *EvtUrl, int *TimeOut, Upnp_SID& SubsId)
 {
     int retVal;
     struct Handle_Info *SInfo = nullptr;
@@ -1662,8 +1671,7 @@ exit_function:
 
 
 #ifdef INCLUDE_CLIENT_APIS
-int UpnpRenewSubscription(UpnpClient_Handle Hnd, int *TimeOut,
-                          const Upnp_SID& SubsId)
+int UpnpRenewSubscription(UpnpClient_Handle Hnd, int *TimeOut, const Upnp_SID& SubsId)
 {
     struct Handle_Info *SInfo = nullptr;
     int retVal;
@@ -1698,9 +1706,8 @@ exit_function:
 #endif /* INCLUDE_CLIENT_APIS */
 
 #ifdef INCLUDE_DEVICE_APIS
-int UpnpNotify(
-    UpnpDevice_Handle Hnd, const char *DevID, const char *ServName,
-    const char **VarName, const char **NewVal, int cVariables)
+int UpnpNotify(UpnpDevice_Handle Hnd, const char *DevID, const char *ServName,
+               const char **VarName, const char **NewVal, int cVariables)
 {
     struct Handle_Info *SInfo = nullptr;
     int retVal;
@@ -1775,18 +1782,16 @@ int UpnpAcceptSubscription(
     }
 
     HandleUnlock();
-    ret = genaInitNotifyVars(Hnd, const_cast<char*>(DevID), const_cast<char*>(ServName), const_cast<char**>(VarName),
-                             const_cast<char**>(NewVal), cVariables, SubsId);
+    ret = genaInitNotifyVars(
+        Hnd, const_cast<char*>(DevID), const_cast<char*>(ServName), const_cast<char**>(VarName),
+        const_cast<char**>(NewVal), cVariables, SubsId);
 
-    UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-               "UpnpAcceptSubscription, ret = %d\n", ret);
+    UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__, "UpnpAcceptSubscription, ret = %d\n", ret);
     return ret;
 }
 
-int UpnpAcceptSubscriptionXML(
-    UpnpDevice_Handle Hnd, const char *DevID, const char *ServName,
-    const std::string& propertyset,
-    const Upnp_SID& SubsId)
+int UpnpAcceptSubscriptionXML(UpnpDevice_Handle Hnd, const char *DevID, const char *ServName,
+                              const std::string& propertyset, const Upnp_SID& SubsId)
 {
     int ret = 0;
     struct Handle_Info *SInfo = nullptr;
@@ -1808,8 +1813,7 @@ int UpnpAcceptSubscriptionXML(
     ret = genaInitNotifyXML(
         Hnd, const_cast<char*>(DevID), const_cast<char*>(ServName), propertyset, SubsId);
 
-    UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-               "UpnpAcceptSubscriptionXML, ret = %d\n", ret);
+    UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__, "UpnpAcceptSubscriptionXML, ret = %d\n", ret);
     return ret;
 }
 
@@ -1892,41 +1896,29 @@ int UpnpDownloadUrlItem(const std::string& url,
 /* Get callback function ptr from a handle. */
 [[maybe_unused]] static Upnp_FunPtr GetCallBackFn(UpnpClient_Handle Hnd)
 {
-    return (static_cast<struct Handle_Info *>(HandleTable[Hnd]))->Callback;
+    return HandleTable[Hnd]->Callback;
 }
 
-/* Assumes at most one client */
-Upnp_Handle_Type GetClientHandleInfo(
-    UpnpClient_Handle *client_handle_out,
-    struct Handle_Info **HndInfo)
+/* We ensure at most one client in UpnpRegisterClient(). Find it. This code is ridiculous, as long
+   as we ensure a single client, the data should be stored in a well-known singleton. The previous
+   version was even weirder, assuming that the entry would always be 1 or 2 (probably dating from
+   the time where a single device could register). Things worked because few or no apps have both
+   clients and devices . */
+Upnp_Handle_Type GetClientHandleInfo(UpnpClient_Handle *hnd, struct Handle_Info **HndInfo)
 {
-    Upnp_Handle_Type ret = HND_CLIENT;
-    UpnpClient_Handle client;
-
-    switch (GetHandleInfo(1, HndInfo)) {
-    case HND_CLIENT:
-        client = 1;
-        break;
-    default:
-        switch (GetHandleInfo(2, HndInfo)) {
-        case HND_CLIENT:
-            client = 2;
-            break;
-        default:
-            client = -1;
-            ret = HND_INVALID;
+    for (int idx = 1; idx < NUM_HANDLE; idx++) {
+        if (HandleTable[idx] != nullptr && HandleTable[idx]->HType == HND_CLIENT) {
+            *HndInfo = HandleTable[idx];
+            *hnd = idx;
+            return HND_CLIENT;
         }
     }
-
-    *client_handle_out = client;
-    return ret;
+    return HND_INVALID;
 }
 
-
+// This code is used to walk all the devices for generating SSDP discovery messages
 Upnp_Handle_Type GetDeviceHandleInfo(
-    UpnpDevice_Handle start,
-    UpnpDevice_Handle *device_handle_out,
-    struct Handle_Info **HndInfo)
+    UpnpDevice_Handle start, UpnpDevice_Handle *device_handle_out, struct Handle_Info **HndInfo)
 {
 #ifdef INCLUDE_DEVICE_APIS
     if (start < 0 || start >= NUM_HANDLE-1) {
@@ -1935,8 +1927,7 @@ Upnp_Handle_Type GetDeviceHandleInfo(
     }
     ++start;
     /* Find it. */
-    for (*device_handle_out=start; *device_handle_out < NUM_HANDLE;
-         (*device_handle_out)++) {
+    for (*device_handle_out=start; *device_handle_out < NUM_HANDLE; (*device_handle_out)++) {
         switch (GetHandleInfo(*device_handle_out, HndInfo)) {
         case HND_DEVICE:
             return HND_DEVICE;
@@ -1951,7 +1942,7 @@ Upnp_Handle_Type GetDeviceHandleInfo(
 }
 
 
-/* Check if we've got a registered device of the address family specified. */
+// This is used in SOAP and GENA to find the device associated with an incoming URL.
 Upnp_Handle_Type GetDeviceHandleInfoForPath(
     const std::string& path, UpnpDevice_Handle *devhdl,
     struct Handle_Info **HndInfo, service_info **serv_info)
@@ -1964,10 +1955,8 @@ Upnp_Handle_Type GetDeviceHandleInfoForPath(
     for (int idx = 1; idx < NUM_HANDLE;    idx++) {
         Handle_Info *hinf;
         if (GetHandleInfo(idx, &hinf) == HND_DEVICE) {
-            if ((*serv_info = FindServiceControlURLPath(
-                     &hinf->ServiceTable,    path)) ||
-                (*serv_info = FindServiceEventURLPath(
-                    &hinf->ServiceTable,  path))) {
+            if ((*serv_info = FindServiceControlURLPath(hinf->serviceTable, path)) ||
+                (*serv_info = FindServiceEventURLPath(hinf->serviceTable, path))) {
                 *HndInfo = hinf;
                 *devhdl = idx;
                 return HND_DEVICE;
@@ -1980,46 +1969,38 @@ Upnp_Handle_Type GetDeviceHandleInfoForPath(
 }
 
 
-Upnp_Handle_Type GetHandleInfo(UpnpClient_Handle Hnd,
-                               struct Handle_Info **HndInfo)
+Upnp_Handle_Type GetHandleInfo(UpnpClient_Handle Hnd, struct Handle_Info **HndInfo)
 {
-    Upnp_Handle_Type ret = HND_INVALID;
-
     if (Hnd < 1 || Hnd >= NUM_HANDLE) {
-        UpnpPrintf(UPNP_ERROR, API, __FILE__, __LINE__,
-                   "GetHandleInfo: out of range\n");
-    } else if (HandleTable[Hnd] == nullptr) {
+        UpnpPrintf(UPNP_ERROR, API, __FILE__, __LINE__, "GetHandleInfo: out of range\n");
+        return HND_INVALID;
+    }
+    if (HandleTable[Hnd] == nullptr) {
         // Don't print anything, we sometimes walk the table
-        //UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-        //           "GetHandleInfo: HTable[%d] is NULL\n",
-        //           Hnd);
-    } else if (HandleTable[Hnd] != nullptr) {
-        *HndInfo = static_cast<struct Handle_Info *>(HandleTable[Hnd]);
-        ret = (*HndInfo)->HType;
+        //UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__, "GetHandleInfo: HTable[%d] is NULL\n", Hnd);
+        return HND_INVALID;
     }
 
-    return ret;
+    if (HndInfo) 
+        *HndInfo = HandleTable[Hnd];
+    return HandleTable[Hnd]->HType;
 }
 
 int PrintHandleInfo(UpnpClient_Handle Hnd)
 {
-    struct Handle_Info * HndInfo;
-    if (HandleTable[Hnd] != nullptr) {
-        HndInfo = static_cast<struct Handle_Info*>(HandleTable[Hnd]);
-        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-                   "Handle_%d Type_%d: \n", Hnd, HndInfo->HType);
-#ifdef INCLUDE_DEVICE_APIS
-        switch(HndInfo->HType) {
-        case HND_CLIENT:
-            break;
-        default:
-            UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-                       "DescURL: %s\n", HndInfo->DescURL);
-        }
-#endif /* INCLUDE_DEVICE_APIS */
-    } else {
+    if (!HandleTable[Hnd])
         return UPNP_E_INVALID_HANDLE;
+
+    auto HndInfo = HandleTable[Hnd];
+    UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__, "Handle_%d Type_%d: \n", Hnd, HndInfo->HType);
+#ifdef INCLUDE_DEVICE_APIS
+    switch (HndInfo->HType) {
+    case HND_CLIENT:
+        break;
+    default:
+        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__, "DescURL: %s\n", HndInfo->DescURL);
     }
+#endif /* INCLUDE_DEVICE_APIS */
     return UPNP_E_SUCCESS;
 }
 
@@ -2038,8 +2019,7 @@ int UpnpSetWebServerRootDir(const char *rootDir)
 #endif /* INTERNAL_WEB_SERVER */
 
 
-int UpnpAddVirtualDir(const char *dirname, const void *cookie,
-                      const void **oldcookie)
+int UpnpAddVirtualDir(const char *dirname, const void *cookie, const void **oldcookie)
 {
     if(UpnpSdkInit != 1) {
         /* SDK is not initialized */
