@@ -48,28 +48,17 @@
 
 #include "miniserver.h"
 
-#include "httputils.h"
-#include "ssdplib.h"
-#include "statcodes.h"
 #include "ThreadPool.h"
-#include "upnpapi.h"
 #include "genut.h"
+#include "ssdplib.h"
+#include "upnpapi.h"
 #include "uri.h"
-#include "netif.h"
-#include "upnp.h"
 
-#include <cassert>
 #include <cerrno>
 #include <chrono>
-#include <cstdio>
-#include <cstdlib>
-#include <cstring>
-#include <iostream>
-#include <sys/types.h>
-#include <stdarg.h>
-#include <thread>
-#include <algorithm>
 #include <condition_variable>
+#include <cstdio>
+#include <cstring>
 
 #include <microhttpd.h>
 
@@ -83,7 +72,7 @@
 
 #define APPLICATION_LISTENING_PORT 49152
 
-using MiniServerState = enum {
+enum MiniServerState{
     MSERV_IDLE,
     MSERV_RUNNING
 };
@@ -104,7 +93,7 @@ static MiniServerCallback gGenaCallback = nullptr;
 
 void SetHTTPGetCallback(MiniServerCallback callback)
 {
-    std::lock_guard<std::mutex> lck(gMServStateMutex);
+    std::scoped_lock lck(gMServStateMutex);
     gGetCallback = callback;
 }
 
@@ -122,7 +111,7 @@ void SetGenaCallback(MiniServerCallback callback)
 
 #endif /* INTERNAL_WEB_SERVER */
 
-static UPNP_INLINE void fdset_if_valid(SOCKET sock, fd_set *set)
+static void fdset_if_valid(SOCKET sock, fd_set* set)
 {
     if (sock != INVALID_SOCKET) {
         FD_SET(sock, set);
@@ -167,8 +156,7 @@ static MHD_Result queryvalues_cb(void *cls, enum MHD_ValueKind,
     return MHD_YES;
 }
 
-// Use int not enum as data.second spares a map code instanciation (at least with some compilers)
-static const std::map<std::string, int> strmethtometh {
+static const std::map<std::string_view, int> strmethtometh {
     {"get", HTTPMETHOD_GET},
     {"head", HTTPMETHOD_HEAD},
     {"m-post", HTTPMETHOD_MPOST},
@@ -177,16 +165,12 @@ static const std::map<std::string, int> strmethtometh {
     {"post", HTTPMETHOD_POST},
     {"subscribe", HTTPMETHOD_SUBSCRIBE},
     {"unsubscribe", HTTPMETHOD_UNSUBSCRIBE},
-        };
+};
 
-static void request_completed_cb(
-    void*, MHD_Connection*,
-    void** con_cls, MHD_RequestTerminationCode)
+static void request_completed_cb(void*, MHD_Connection*, MHDTransaction** con_cls, MHD_RequestTerminationCode)
 {
-    if (nullptr == con_cls)
-        return;
-    auto mhdt = static_cast<MHDTransaction *>(*con_cls);
-    delete mhdt;
+    if (con_cls)
+        delete *con_cls;
 }
 
 
@@ -212,7 +196,7 @@ static MHD_Result filter_connections(
 // exploit implemented in a local browser. Of course this supposes that the
 // browser will actually set a meaningful HOST, which they appear to do.
 enum VHH_Status{VHH_YES, VHH_NO, VHH_REDIRECT};
-static VHH_Status validate_host_header(MHDTransaction *mhdt, NetIF::IPAddr& claddr)
+static VHH_Status validate_host_header(MHDTransaction* mhdt, const NetIF::IPAddr& claddr)
 {
     // Find HOST header
     auto hostit = mhdt->headers.find("host");
@@ -285,10 +269,10 @@ static VHH_Status validate_host_header(MHDTransaction *mhdt, NetIF::IPAddr& clad
 }
 
 static std::string rebuild_url_from_mhdt(
-    MHDTransaction *mhdt, const std::string& path, NetIF::IPAddr& claddr)
+    MHDTransaction* mhdt, const std::string& path, const NetIF::IPAddr& claddr)
 {
     std::string aurl("http://");
-    auto hostport = UpnpGetUrlHostPortForClient(mhdt->client_address);
+    auto hostport = UpnpGetUrlHostPortForClient(&mhdt->client_address);
     if (hostport.empty()) {
         UpnpPrintf(UPNP_INFO, MSERV, __FILE__, __LINE__,
                    "answer_to_connection: got empty hostport for connection from %s\n",
@@ -299,8 +283,8 @@ static std::string rebuild_url_from_mhdt(
     aurl += path;
     if (!mhdt->queryvalues.empty()) {
         aurl += "?";
-        for (const auto& entry: mhdt->queryvalues) {
-            aurl += query_encode(entry.first) + "=" + query_encode(entry.second) + "&";
+        for (const auto& [q, v] : mhdt->queryvalues) {
+            aurl += query_encode(q) + "=" + query_encode(v) + "&";
         }
         aurl.pop_back();
     }
@@ -321,9 +305,8 @@ static MHD_Result answer_to_connection(
         auto mhdt = new MHDTransaction;
         *con_cls = mhdt;
         MHD_get_connection_values(conn, MHD_HEADER_KIND, headers_cb, mhdt);
-        mhdt->client_address =
-            reinterpret_cast<struct sockaddr_storage*>(
-                MHD_get_connection_info (conn,MHD_CONNECTION_INFO_CLIENT_ADDRESS)->client_addr);
+        auto ca = MHD_get_connection_info(conn, MHD_CONNECTION_INFO_CLIENT_ADDRESS)->client_addr;
+        mhdt->copyToClientAddress(ca);
 
         MHD_get_connection_values(conn, MHD_GET_ARGUMENT_KIND, queryvalues_cb, mhdt);
         mhdt->conn = conn;
@@ -349,7 +332,7 @@ static MHD_Result answer_to_connection(
             return MHD_YES;
         }
         
-        NetIF::IPAddr claddr(reinterpret_cast<sockaddr*>(mhdt->client_address));
+        NetIF::IPAddr claddr(reinterpret_cast<sockaddr*>(&mhdt->client_address));
         switch (validate_host_header(mhdt, claddr)) {
         case VHH_YES: return MHD_YES;
         case VHH_NO: return MHD_NO;
@@ -362,7 +345,8 @@ static MHD_Result answer_to_connection(
             return MHD_NO;
         }
         UpnpPrintf(UPNP_INFO, MSERV, __FILE__, __LINE__, "Redirecting to [%s]\n", aurl.c_str());
-        struct MHD_Response *response = MHD_create_response_from_buffer(0,nullptr,MHD_RESPMEM_PERSISTENT);
+        struct MHD_Response *response =
+            MHD_create_response_from_buffer(0,nullptr,MHD_RESPMEM_PERSISTENT);
         if (nullptr == response ) {
             UpnpPrintf(UPNP_DEBUG, MSERV, __FILE__, __LINE__,
                        "answer_to_connection: can't create redirect\n");
@@ -403,7 +387,7 @@ static MHD_Result answer_to_connection(
     case HTTPMETHOD_POST:
     case HTTPMETHOD_HEAD:
     {
-        std::lock_guard<std::mutex> lck(gMServStateMutex);
+        std::scoped_lock lck(gMServStateMutex);
         callback = gGetCallback;
         break;
     }
@@ -468,8 +452,6 @@ static int receive_from_stopSock(SOCKET ssock, fd_set *set)
 
 class MiniServerJobWorker : public JobWorker {
 public:
-    MiniServerJobWorker() = default;
-    ~MiniServerJobWorker() override = default;
     void work() override;
 };
 /*!
@@ -485,7 +467,7 @@ void MiniServerJobWorker::work()
     fd_set expSet;
     fd_set rdSet;
     SOCKET maxMiniSock;
-    int ret = 0;
+    int ret;
     int stopSock = 0;
 
     maxMiniSock = 0;
@@ -503,7 +485,7 @@ void MiniServerJobWorker::work()
     ++maxMiniSock;
 
     {
-        std::unique_lock<std::mutex> lck(gMServStateMutex);
+        std::scoped_lock lck(gMServStateMutex);
         gMServState = MSERV_RUNNING;
         gMServStateCV.notify_all();
     }
@@ -545,11 +527,10 @@ void MiniServerJobWorker::work()
             ssdp_read(miniSocket->ssdpSock6, &rdSet);
             ssdp_read(miniSocket->ssdpSock6UlaGua, &rdSet);
         }
-        stopSock = receive_from_stopSock(
-            miniSocket->miniServerStopSock, &rdSet);
+        stopSock = receive_from_stopSock(miniSocket->miniServerStopSock, &rdSet);
     }
 
-    std::unique_lock<std::mutex> lck(gMServStateMutex);
+    std::scoped_lock lck(gMServStateMutex);
     delete miniSocket;
     miniSocket = nullptr;
     gMServState = MSERV_IDLE;
@@ -583,7 +564,7 @@ static int get_port(
         *port = ntohs(reinterpret_cast<struct sockaddr_in6*>(&sockinfo)->sin6_port);
     }
     UpnpPrintf(UPNP_INFO, MSERV, __FILE__, __LINE__,
-               "sockfd = %d, .... port = %d\n", sockfd, static_cast<int>(*port));
+               "sockfd = %d, .... port = %d\n", static_cast<int>(sockfd), static_cast<int>(*port));
 
     return 0;
 }
@@ -614,7 +595,12 @@ static int get_miniserver_stopsock(MiniServerSockArray *out)
     /* Bind to local socket. */
     stop_sockaddr = {};
     stop_sockaddr.sin_family = static_cast<sa_family_t>(AF_INET);
-    stop_sockaddr.sin_addr.s_addr = inet_addr("127.0.0.1");
+
+    if (inet_pton(AF_INET, "127.0.0.1", &stop_sockaddr.sin_addr) != 1) {
+        UpnpPrintf(UPNP_CRITICAL, MSERV, __FILE__, __LINE__,
+                   "Error in converting IP address\n");
+        return UPNP_E_INVALID_PARAM;
+    }
     ret = bind(out->miniServerStopSock,
                reinterpret_cast<struct sockaddr *>(&stop_sockaddr),
                sizeof(stop_sockaddr));
@@ -692,6 +678,14 @@ static int available_port(int reqport)
     return ret;
 }
 
+/* This enables printf like format checking by the compiler. */
+#ifndef _MSC_VER
+#ifdef __MINGW32__
+[[gnu::format(__MINGW_PRINTF_FORMAT, 2, 0)]]
+#else
+[[gnu::format(printf, 2, 0)]]
+#endif
+#endif // !_MSC_VER
 static void mhdlogger(void *, const char *fmt, va_list ap)
 {
     char buf[1024];
@@ -708,19 +702,17 @@ int StartMiniServer(uint16_t *listen_port4, uint16_t *listen_port6)
     unsigned int mhdflags = 0;
 
     {
-        std::unique_lock<std::mutex> lck(gMServStateMutex);
+        std::scoped_lock lck(gMServStateMutex);
         if (gMServState != MSERV_IDLE) {
             /* miniserver running. */
-            UpnpPrintf(UPNP_CRITICAL, MSERV, __FILE__, __LINE__,
-                       "miniserver: ALREADY RUNNING !\n");
+            UpnpPrintf(UPNP_CRITICAL, MSERV, __FILE__, __LINE__, "miniserver: ALREADY RUNNING !\n");
             return UPNP_E_INTERNAL_ERROR;
         }
     }
 
     miniSocket = new MiniServerSockArray;
     if (nullptr == miniSocket) {
-        UpnpPrintf(UPNP_CRITICAL, MSERV, __FILE__, __LINE__,
-                   "miniserver: OUT OF MEMORY !\n");
+        UpnpPrintf(UPNP_CRITICAL, MSERV, __FILE__, __LINE__, "miniserver: OUT OF MEMORY !\n");
         return UPNP_E_OUTOF_MEMORY;
     }
 
@@ -732,8 +724,18 @@ int StartMiniServer(uint16_t *listen_port4, uint16_t *listen_port6)
         goto out;
     }
 
+    /* Check what port we should listen on */
+    port = available_port(static_cast<int>(*listen_port4));
+    if (port < 0) {
+        UpnpPrintf(UPNP_CRITICAL, MSERV, __FILE__, __LINE__,
+                   "miniserver: available_port() failed !\n");
+        return port;
+    }
+    *listen_port4 = port;
+    *listen_port6 = port;
+
     /* SSDP socket for discovery/advertising. */
-    ret_code = get_ssdp_sockets(miniSocket);
+    ret_code = get_ssdp_sockets(miniSocket, port);
     if (ret_code != UPNP_E_SUCCESS) {
         UpnpPrintf(UPNP_CRITICAL, MSERV, __FILE__, __LINE__,
                    "miniserver: get_ssdp_sockets() failed\n");
@@ -760,18 +762,7 @@ int StartMiniServer(uint16_t *listen_port4, uint16_t *listen_port6)
     }
     
 #ifdef INTERNAL_WEB_SERVER
-    port = available_port(static_cast<int>(*listen_port4));
-    if (port < 0) {
-        UpnpPrintf(UPNP_CRITICAL, MSERV, __FILE__, __LINE__,
-                   "miniserver: available_port() failed !\n");
-        return port;
-    }
-    *listen_port4 = port;
-    *listen_port6 = port;
-
-    mhdflags = MHD_USE_THREAD_PER_CONNECTION |
-        MHD_USE_INTERNAL_POLLING_THREAD |
-        MHD_USE_DEBUG;
+    mhdflags = MHD_USE_THREAD_PER_CONNECTION | MHD_USE_INTERNAL_POLLING_THREAD | MHD_USE_DEBUG;
 
 #ifdef UPNP_ENABLE_IPV6
     if (using_ipv6()) {
@@ -784,7 +775,7 @@ int StartMiniServer(uint16_t *listen_port4, uint16_t *listen_port6)
         filter_connections, nullptr, /* Accept policy callback and arg */
         &answer_to_connection, nullptr, /* Request handler and arg */
         MHD_OPTION_NOTIFY_COMPLETED, request_completed_cb, nullptr,
-        MHD_OPTION_CONNECTION_TIMEOUT, static_cast<unsigned int>(UPNP_TIMEOUT),
+        MHD_OPTION_CONNECTION_TIMEOUT, static_cast<unsigned int>(HTTP_DEFAULT_TIMEOUT),
         MHD_OPTION_EXTERNAL_LOGGER, mhdlogger, nullptr, 
         MHD_OPTION_END);
     if (nullptr == mhd) {
@@ -797,8 +788,7 @@ int StartMiniServer(uint16_t *listen_port4, uint16_t *listen_port6)
 
 out:
     if (ret_code != UPNP_E_SUCCESS) {
-        UpnpPrintf(UPNP_CRITICAL, MSERV, __FILE__, __LINE__,
-                   "startminiserver failed\n");
+        UpnpPrintf(UPNP_CRITICAL, MSERV, __FILE__, __LINE__, "startminiserver failed\n");
         delete miniSocket;
         miniSocket = nullptr;
     }
@@ -819,6 +809,10 @@ int StopMiniServer()
         return 0;
     }
 
+#ifdef INTERNAL_WEB_SERVER
+    MHD_stop_daemon(mhd);
+#endif
+
     sock = socket(AF_INET, SOCK_DGRAM, 0);
     if (sock == INVALID_SOCKET) {
         posix_strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -827,9 +821,15 @@ int StopMiniServer()
         return 0;
     }
     stopaddr.sin_family = static_cast<sa_family_t>(AF_INET);
-    stopaddr.sin_addr.s_addr = inet_addr("127.0.0.1");
     stopaddr.sin_port = htons(miniSocket->stopPort);
 
+    if (inet_pton(AF_INET, "127.0.0.1", &stopaddr.sin_addr) != 1) {
+        UpnpPrintf(UPNP_INFO, SSDP, __FILE__, __LINE__,
+                   "Error in converting IP address\n");
+        UpnpCloseSocket(sock);
+        return 0;
+    }
+
     while (gMServState != MSERV_IDLE) {
         sendto(sock, buf, bufLen, 0,
                reinterpret_cast<struct sockaddr *>(&stopaddr), socklen);
