@@ -35,29 +35,28 @@
 #ifdef INCLUDE_DEVICE_APIS
 #if EXCLUDE_SSDP == 0
 
+#include "ThreadPool.h"
+#include "genut.h"
 #include "httputils.h"
 #include "ssdplib.h"
 #include "statcodes.h"
-#include "ThreadPool.h"
 #include "upnpapi.h"
-#include "UpnpInet.h"
-#include "upnpdebug.h"
-#include "TimerThread.h"
-#include "genut.h"
-#include "inet_pton.h"
-#include "netif.h"
 
-#include <cassert>
+#include <algorithm>
+#include <chrono>
 #include <cstdio>
 #include <cstring>
 #include <iostream>
 #include <sstream>
 #include <string>
 #include <thread>
-#include <algorithm>
-#include <chrono>
 
 struct SsdpSearchReply {
+    SsdpSearchReply(int a, UpnpDevice_Handle h, const sockaddr_storage* da, SsdpEntity e)
+        : MaxAge(a), handle(h), event(std::move(e))
+    {
+        std::memcpy(&dest_addr, da, sizeof(dest_addr));
+    }
     int MaxAge;
     UpnpDevice_Handle handle;
     struct sockaddr_storage dest_addr;
@@ -73,63 +72,50 @@ struct SSDPPwrState {
 // A bundle to simplify arg lists
 struct SSDPCommonData {
     SOCKET sock;
-    struct sockaddr *DestAddr;
-    const char *DevOrServType;
+    struct sockaddr_storage *DestAddr;
     SSDPPwrState pwr;
     std::string prodvers;
 };
 
 class SSDPSearchJobWorker : public JobWorker {
 public:
-    explicit SSDPSearchJobWorker(SsdpSearchReply *reply)
-        : m_reply(reply) {}
-    ~SSDPSearchJobWorker() override {
-        delete m_reply;
-    }
-    SSDPSearchJobWorker(const SSDPSearchJobWorker&) = delete;
-    SSDPSearchJobWorker& operator=(const SSDPSearchJobWorker&) = delete;
+    explicit SSDPSearchJobWorker(std::unique_ptr<SsdpSearchReply> reply)
+        : m_reply(std::move(reply)) {}
     void work() override {
         AdvertiseAndReply(m_reply->handle, MSGTYPE_REPLY,
                           m_reply->MaxAge,
-                          reinterpret_cast<struct sockaddr *>(&m_reply->dest_addr),
+                          &m_reply->dest_addr,
                           m_reply->event);
     }
-    SsdpSearchReply *m_reply;
+    std::unique_ptr<SsdpSearchReply> m_reply;
 };
 
-void ssdp_handle_device_request(SSDPPacketParser& parser,
-                                struct sockaddr_storage *dest_addr)
+void ssdp_handle_device_request(const SSDPPacketParser& parser, struct sockaddr_storage *dest_addr)
 {
     int handle, start;
     struct Handle_Info *dev_info = nullptr;
-    int mx;
     SsdpEntity event;
-    int maxAge;
-
+    
     /* check man hdr. */
     if (!parser.man || strcmp(parser.man, R"("ssdp:discover")") != 0) {
         /* bad or missing hdr. */
-        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-                   "ssdp_handle_device_req: no/bad MAN header\n");
+        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__, "ssdp_handle_device_req: no/bad MAN header\n");
         return;
     }
-    /* MX header. Must be >= 1*/
-    if (!parser.mx || (mx = atoi(parser.mx)) <= 0) {
-        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-                   "ssdp_handle_device_req: no/bad MX header\n");
-        return;
+    /* MX header. May be absent for a unicast request. Consistency has been checked in ssdp_server */
+    int mx = 0;
+    if (parser.mx) {
+        mx = atoi(parser.mx);
     }
     /* ST header. */
     if (!parser.st || ssdp_request_type(parser.st, &event) == -1) {
-        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-                   "ssdp_handle_device_req: no/bad ST header\n");
+        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__, "ssdp_handle_device_req: no/bad ST header\n");
         return;
     }
 
-    // Loop to dispatch the packet to each of our configured devices
-    // by starting the handle search at the last processed
-    // position. each device response is scheduled by the ThreadPool
-    // with a random delay based on the MX header of the search packet.
+    // Loop to dispatch the packet to each of our configured devices by starting the handle search
+    // at the last processed position. each device response is scheduled by the ThreadPool with a
+    // random delay based on the MX header of the search packet.
     start = 0;
     for (;;) {
         HandleLock();
@@ -142,13 +128,11 @@ void ssdp_handle_device_request(SSDPPacketParser& parser,
             /* no info found. */
             return;
         }
-        maxAge = dev_info->MaxAge;
+        int maxAge = dev_info->MaxAge;
         HandleUnlock();
 
-        UpnpPrintf(UPNP_DEBUG, API, __FILE__, __LINE__,
-                   "MAX-AGE        =  %d\n", maxAge);
-        UpnpPrintf(UPNP_DEBUG, API, __FILE__, __LINE__,
-                   "MX       =  %d\n", maxAge);
+        UpnpPrintf(UPNP_DEBUG, API, __FILE__, __LINE__, "MAX-AGE        =  %d\n", maxAge);
+        UpnpPrintf(UPNP_DEBUG, API, __FILE__, __LINE__, "MX       =  %d\n", mx);
         UpnpPrintf(UPNP_DEBUG, API, __FILE__, __LINE__,
                    "DeviceType     =    %s\n", event.DeviceType.c_str());
         UpnpPrintf(UPNP_DEBUG, API, __FILE__, __LINE__,
@@ -156,20 +140,19 @@ void ssdp_handle_device_request(SSDPPacketParser& parser,
         UpnpPrintf(UPNP_DEBUG, API, __FILE__, __LINE__,
                    "ServiceType =  %s\n", event.ServiceType.c_str());
 
-        auto threadArg = new SsdpSearchReply;
-        threadArg->handle = handle;
-        memcpy(&threadArg->dest_addr, dest_addr, sizeof(threadArg->dest_addr));
-        threadArg->event = event;
-        threadArg->MaxAge = maxAge;
-
-        mx = std::max(1, mx);
-        /* Subtract a bit from the mx to allow for network/processing delays */
-        int delayms = rand() %    (mx * 1000 - 100);
-        UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
-                   "ssdp_handle_device_req: scheduling resp in %d ms\n",delayms);
-        auto worker = std::make_unique<SSDPSearchJobWorker>(threadArg);
-        gTimerThread->schedule(TimerThread::SHORT_TERM, std::chrono::milliseconds(delayms),
-                               nullptr, std::move(worker));
+        auto threadArg = std::make_unique<SsdpSearchReply>(maxAge, handle, dest_addr, event);
+        auto worker = std::make_unique<SSDPSearchJobWorker>(std::move(threadArg));
+        if (mx) {
+            mx = std::max(1, mx);
+            /* Subtract a bit from the mx to allow for network/processing delays */
+            int delayms = rand() % (mx * 1000 - 100);
+            UpnpPrintf(UPNP_ALL, API, __FILE__, __LINE__,
+                       "ssdp_handle_device_req: scheduling resp in %d ms\n", delayms);
+            gTimerThread->schedule(TimerThread::SHORT_TERM, std::chrono::milliseconds(delayms),
+                                   nullptr, std::move(worker));
+        } else {
+            gSendThreadPool.addJob(std::move(worker));
+        }
         start = handle;
     }
 }
@@ -275,7 +258,7 @@ error:
 }
 
 static SOCKET createReplySocket6(
-    struct sockaddr_in6 *destaddr, std::string& lochost)
+    const struct sockaddr_in6 *destaddr, std::string& lochost)
 {
     SOCKET sock = socket(AF_INET6, SOCK_DGRAM, 0);
     if (sock == INVALID_SOCKET) {
@@ -333,17 +316,18 @@ static bool ssdpMcastAddr(struct sockaddr_storage& ss, int AddressFamily)
     return true;
 }
 
-static int sendPackets(SOCKET sock, struct sockaddr *daddr, int cnt, std::string *pckts)
+static int sendPackets(SOCKET sock, struct sockaddr_storage *daddr, int cnt, std::string *pckts)
 {
-    NetIF::IPAddr destip(daddr);
-    int socklen = daddr->sa_family == AF_INET ? sizeof(struct sockaddr_in) :
+    NetIF::IPAddr destip(reinterpret_cast<struct sockaddr*>(daddr));
+    int socklen = daddr->ss_family == AF_INET ? sizeof(struct sockaddr_in) :
         sizeof(struct sockaddr_in6);
 
     for (int i = 0; i < cnt; i++) {
         ssize_t rc;
         UpnpPrintf(UPNP_DEBUG, SSDP, __FILE__, __LINE__, ">>> SSDP SEND to %s >>>\n%s\n",
                    destip.straddr().c_str(), pckts[i].c_str());
-        rc = sendto(sock, pckts[i].c_str(), pckts[i].size(), 0, daddr, socklen);
+        rc = sendto(sock, pckts[i].c_str(), pckts[i].size(), 0,
+                    reinterpret_cast<struct sockaddr*>(daddr), socklen);
 
         if (rc == -1) {
             char errorBuffer[ERROR_BUFFER_LEN];
@@ -360,7 +344,7 @@ static int sendPackets(SOCKET sock, struct sockaddr *daddr, int cnt, std::string
 static void CreateServicePacket(
     SSDPDevMessageType msg_type, const char *nt, const char *usn,
     const std::string& location, int duration, std::string &packet,
-    int AddressFamily, SSDPPwrState& pwr, const std::string& prodvers)
+    int AddressFamily, const SSDPPwrState& pwr, const std::string& prodvers)
 {
     std::ostringstream str;
     switch (msg_type) {
@@ -421,13 +405,15 @@ static void CreateServicePacket(
             "SleepPeriod: " << pwr.SleepPeriod << "\r\n" <<
             "RegistrationState: " << pwr.RegistrationState << "\r\n";
     }
+    str << "BOOTID.UPNP.ORG: " << g_bootidUpnpOrg << "\r\n" <<
+        "CONFIGID.UPNP.ORG: " << g_configidUpnpOrg << "\r\n";
 
     str << "\r\n";
     packet = str.str();
 }
 
 static int DeviceAdvertisementOrShutdown(
-    SSDPCommonData& sscd, SSDPDevMessageType msgtype, const char *DevType,
+    const SSDPCommonData& sscd, SSDPDevMessageType msgtype, const char *DevType,
     int RootDev, const char *Udn, const std::string& Location, int Duration)
 {
     char Mil_Usn[LINE_SIZE];
@@ -442,17 +428,17 @@ static int DeviceAdvertisementOrShutdown(
             goto error_handler;
         CreateServicePacket(msgtype, "upnp:rootdevice",
                             Mil_Usn, Location, Duration, msgs[0],
-                            sscd.DestAddr->sa_family, sscd.pwr, sscd.prodvers);
+                            sscd.DestAddr->ss_family, sscd.pwr, sscd.prodvers);
     }
     /* both root and sub-devices need to send these two messages */
     CreateServicePacket(msgtype, Udn, Udn,
-                        Location, Duration, msgs[1], sscd.DestAddr->sa_family,
+                        Location, Duration, msgs[1], sscd.DestAddr->ss_family,
                         sscd.pwr, sscd.prodvers);
     rc = snprintf(Mil_Usn, sizeof(Mil_Usn), "%s::%s", Udn, DevType);
     if (rc < 0 || static_cast<unsigned int>(rc) >= sizeof(Mil_Usn))
         goto error_handler;
     CreateServicePacket(msgtype, DevType, Mil_Usn, Location, Duration, msgs[2],
-                        sscd.DestAddr->sa_family, sscd.pwr, sscd.prodvers);
+                        sscd.DestAddr->ss_family, sscd.pwr, sscd.prodvers);
     /* check error */
     if ((RootDev && msgs[0].empty()) || msgs[1].empty() || msgs[2].empty()) {
         goto error_handler;
@@ -471,7 +457,7 @@ error_handler:
 }
 
 static int SendReply(
-    SSDPCommonData& sscd, const char *DevType, int RootDev,
+    const SSDPCommonData& sscd, const char *DevType, int RootDev,
     const char *Udn, const std::string& Location, int Duration, int ByType)
 {
     int ret_code = UPNP_E_OUTOF_MEMORY;
@@ -480,7 +466,7 @@ static int SendReply(
     char Mil_Usn[LINE_SIZE];
     int i;
     int rc = 0;
-    auto family = static_cast<int>(sscd.DestAddr->sa_family);
+    auto family = static_cast<int>(sscd.DestAddr->ss_family);
 
     if (RootDev) {
         /* one msg for root device */
@@ -521,13 +507,13 @@ error_handler:
 }
 
 static int DeviceReply(
-    SSDPCommonData& sscd, const char *DevType, int RootDev,
+    const SSDPCommonData& sscd, const char *DevType, int RootDev,
     const char *Udn, const std::string& Location, int Duration)
 {
     std::string szReq[3];
     char Mil_Nt[LINE_SIZE], Mil_Usn[LINE_SIZE];
     int rc = 0;
-    auto family = static_cast<int>(sscd.DestAddr->sa_family);
+    auto family = static_cast<int>(sscd.DestAddr->ss_family);
 
     /* create 2 or 3 msgs */
     if (RootDev) {
@@ -567,7 +553,7 @@ static int DeviceReply(
 }
 
 static int ServiceSend(
-    SSDPCommonData& sscd, SSDPDevMessageType tp, const char *ServType,
+    const SSDPCommonData& sscd, SSDPDevMessageType tp, const char *ServType,
     const char *Udn, const std::string& Location, int Duration)
 {
     char Mil_Usn[LINE_SIZE];
@@ -578,7 +564,7 @@ static int ServiceSend(
     if (rc < 0 || static_cast<unsigned int>(rc) >= sizeof(Mil_Usn))
         return UPNP_E_OUTOF_MEMORY;
     CreateServicePacket(tp, ServType, Mil_Usn, Location, Duration, szReq[0],
-                        sscd.DestAddr->sa_family, sscd.pwr, sscd.prodvers);
+                        sscd.DestAddr->ss_family, sscd.pwr, sscd.prodvers);
     if (szReq[0].empty()) {
         return UPNP_E_OUTOF_MEMORY;
 
@@ -622,7 +608,7 @@ static bool sameServOrDevNoVers(const char *his, const char *mine)
 // be subdevices
 static int AdvertiseAndReplyOneDest(
     UpnpDevice_Handle Hnd, SSDPDevMessageType tp, int Exp,
-    struct sockaddr *DestAddr, const SsdpEntity& sdata, SOCKET sock,
+    struct sockaddr_storage *DestAddr, const SsdpEntity& sdata, SOCKET sock,
     const std::string& lochost)
 {
     int retVal = UPNP_E_SUCCESS;
@@ -639,14 +625,7 @@ static int AdvertiseAndReplyOneDest(
     }
 
     int defaultExp = SInfo->MaxAge;
-
-    struct SSDPCommonData sscd;
-    sscd.sock = sock;
-    sscd.DestAddr = DestAddr;
-    sscd.pwr = SSDPPwrState{SInfo->PowerState, SInfo->SleepPeriod, SInfo->RegistrationState};
-    sscd.prodvers = SInfo->productversion;
-
-
+    SSDPCommonData sscd{sock, DestAddr, SSDPPwrState{SInfo->PowerState, SInfo->SleepPeriod, SInfo->RegistrationState}, SInfo->productversion};
     std::string location{SInfo->DescURL};
     replaceLochost(location, lochost);
     std::string lowerloc{SInfo->LowerDescURL};
@@ -798,7 +777,7 @@ static int AdvertiseAndReplyOneDest(
 // work for each of the appropriate network addresses/interfaces.
 
 int AdvertiseAndReply(UpnpDevice_Handle Hnd, SSDPDevMessageType tp, int Exp,
-                      struct sockaddr *repDestAddr, const SsdpEntity& sdata)
+                      struct sockaddr_storage *repDestAddr, const SsdpEntity& sdata)
 {
     bool isNotify = (tp == MSGTYPE_ADVERTISEMENT || tp == MSGTYPE_SHUTDOWN);
     int ret = UPNP_E_SUCCESS;
@@ -812,7 +791,7 @@ int AdvertiseAndReply(UpnpDevice_Handle Hnd, SSDPDevMessageType tp, int Exp,
                        "ssdp_device: mcast for %s\n", netif.getname().c_str());
 
             struct sockaddr_storage dss;
-            auto destaddr = reinterpret_cast<struct sockaddr*>(&dss);
+            auto destaddr = &dss;
 
 #ifdef UPNP_ENABLE_IPV6
             if (using_ipv6()) {
@@ -836,16 +815,13 @@ int AdvertiseAndReply(UpnpDevice_Handle Hnd, SSDPDevMessageType tp, int Exp,
 #endif /* UPNP_ENABLE_IPV6 */
 
             ssdpMcastAddr(dss, AF_INET);
-            auto addresses = netif.getaddresses();
-            for (const auto& ipaddr : addresses.first) {
-                if (ipaddr.family() == NetIF::IPAddr::Family::IPV4) {
-                    const struct sockaddr_storage& fss{ipaddr.getaddr()};
-                    sock = createMulticastSocket4(
-                        reinterpret_cast<const struct sockaddr_in*>(&fss),
-                        lochost);
-                } else {
+            for (const auto& ipaddr : netif.getaddresses().first) {
+                if (ipaddr.family() != NetIF::IPAddr::Family::IPV4)
                     continue;
-                }
+                const struct sockaddr_storage& fss{ipaddr.getaddr()};
+                sock = createMulticastSocket4(
+                    reinterpret_cast<const struct sockaddr_in*>(&fss),
+                    lochost);
                 if (sock == INVALID_SOCKET) {
                     goto exitfunc;
                 }
@@ -857,7 +833,7 @@ int AdvertiseAndReply(UpnpDevice_Handle Hnd, SSDPDevMessageType tp, int Exp,
             }
         }
     } else {
-        sock = repDestAddr->sa_family == AF_INET ?
+        sock = repDestAddr->ss_family == AF_INET ?
             createReplySocket4(
                 reinterpret_cast<struct sockaddr_in*>(repDestAddr), lochost) :
             createReplySocket6(
