@@ -33,10 +33,13 @@
 
 #include "TimerThread.h"
 
+#include <algorithm>
 #include <cassert>
 #include <chrono>
 #include <condition_variable>
+#include <list>
 #include <mutex>
+#include <utility>
 
 using namespace std::chrono;
 
@@ -45,20 +48,22 @@ struct TimerEvent {
     TimerEvent(
         std::unique_ptr<JobWorker> w, ThreadPool::ThreadPriority prio,
         TimerThread::Duration p, system_clock::time_point et, int _id)
-        : priority(prio), persistent(p), eventTime(et), id(_id) {
-        worker = std::move(w);
+        : worker(std::move(w)), eventTime(et), id(_id), priority(prio), persistent(p)
+    {
     }
-    
+
     std::unique_ptr<JobWorker> worker;
-    ThreadPool::ThreadPriority priority;
-    /*! [in] Long term or short term job. */
-    TimerThread::Duration persistent;
     /*! [in] Absolute time for event in seconds since Jan 1, 1970. */
     system_clock::time_point eventTime;
     int id;
+    ThreadPool::ThreadPriority priority;
+    /*! [in] Long term or short term job. */
+    TimerThread::Duration persistent;
 };
 
 
+// This is the worker for the permanent timer thread, in charge of dispatching jobs at appropriate
+// times
 class TimerJobWorker : public JobWorker {
 public:
     explicit TimerJobWorker(TimerThread::Internal* parent)
@@ -74,7 +79,7 @@ public:
     std::mutex mutex;
     std::condition_variable condition;
     int lastEventId{0};
-    std::list<TimerEvent*> eventQ;
+    std::list<TimerEvent> eventQ;
     int inshutdown{0};
     ThreadPool *tp{nullptr};
 };
@@ -82,17 +87,12 @@ public:
 /*!
  * \brief Implements timer thread.
  *
- * Waits for next event to occur and schedules associated job into threadpool.
- *    arg is cast to (TimerThread *).
+ * tl;dr: Sleeps until next event scheduled time, then dispatches it then sleeps...
  */
 void TimerJobWorker::work()
 {
     auto timer = m_parent;
-    TimerEvent *nextEvent = nullptr;
-    system_clock::time_point nextEventTime = system_clock::now();
-
     assert(timer != nullptr);
-
     std::unique_lock<std::mutex> lck(timer->mutex);
 
     while (true) {
@@ -103,27 +103,22 @@ void TimerJobWorker::work()
             timer->condition.notify_all();
             return;
         }
-        nextEvent = nullptr;
+        system_clock::time_point currentTime = system_clock::now();
         /* Get the next event if possible. */
         if (!timer->eventQ.empty()) {
-            nextEvent = timer->eventQ.front();
-            nextEventTime = nextEvent->eventTime;
-        }
-        system_clock::time_point currentTime = system_clock::now();
-        /* If time has elapsed, schedule job. */
-        if (nextEvent && currentTime >= nextEventTime) {
-            if (nextEvent->persistent) {
-                timer->tp->addPersistent(std::move(nextEvent->worker), nextEvent->priority);
+            TimerEvent& nextEvent(timer->eventQ.front());
+            if (currentTime >= nextEvent.eventTime) {
+                /* If time has elapsed, schedule job. */
+                if (timer->eventQ.front().persistent) {
+                    timer->tp->addPersistent(std::move(nextEvent.worker), nextEvent.priority);
+                } else {
+                    timer->tp->addJob(std::move(nextEvent.worker), nextEvent.priority);
+                }
+                timer->eventQ.pop_front();
             } else {
-                timer->tp->addJob(std::move(nextEvent->worker), nextEvent->priority);
+                auto tm = nextEvent.eventTime;
+                timer->condition.wait_until(lck, tm);
             }
-            timer->eventQ.pop_front();
-            delete nextEvent;
-            continue;
-        }
-        if (nextEvent) {
-            auto tm = nextEvent->eventTime;
-            timer->condition.wait_until(lck, tm);
         } else {
             timer->condition.wait(lck);
         }
@@ -132,7 +127,7 @@ void TimerJobWorker::work()
 
 TimerThread::Internal::Internal(ThreadPool *tp)
 {
-    std::unique_lock<std::mutex> lck(mutex);
+    std::scoped_lock lck(mutex);
     this->tp = tp;
     auto worker = std::make_unique<TimerJobWorker>(this);
     tp->addPersistent(std::move(worker), ThreadPool::HIGH_PRIORITY);
@@ -154,36 +149,24 @@ int TimerThread::schedule(
     std::unique_ptr<JobWorker> worker, ThreadPool::ThreadPriority priority
     )
 {
-    std::unique_lock<std::mutex> lck(m->mutex);
-    int rc = EOUTOFMEM;
+    std::scoped_lock lck(m->mutex);
 
-    auto newEvent = new TimerEvent(std::move(worker), priority, persistence, when, m->lastEventId);
-    if (newEvent == nullptr) {
-        return rc;
-    }
     if (id) {
         *id = m->lastEventId;
     }
     /* add job to Q. Q is ordered by eventTime with the head of the Q being
      * the next event. */
-    rc = 0;
-    int found = 0;
-    for (auto it = m->eventQ.begin(); it != m->eventQ.end(); it++) {
-        if ((*it)->eventTime >= when) {
-            m->eventQ.insert(it, newEvent);
-            found = 1;
-            break;
-        }
-    }
-    /* add to the end of Q. */
-    if (!found) {
-        m->eventQ.push_back(newEvent);
-    }
+    auto it = std::find_if(m->eventQ.begin(), m->eventQ.end(),
+                           [=](const auto& e) { return e.eventTime >= when; });
+    if (it != m->eventQ.end())
+        m->eventQ.emplace(it, std::move(worker), priority, persistence, when, m->lastEventId);
+    else
+        m->eventQ.emplace_back(std::move(worker), priority, persistence, when, m->lastEventId);
+
     /* signal change in Q. */
     m->condition.notify_all();
     m->lastEventId++;
-
-    return rc;
+    return 0;
 }
 
 int TimerThread::schedule(
@@ -212,19 +195,16 @@ int TimerThread::schedule(
 
 int TimerThread::remove(int id)
 {
-    int rc = -1;
-    std::unique_lock<std::mutex> lck(m->mutex);
+    std::scoped_lock lck(m->mutex);
 
-    for (auto it = m->eventQ.begin(); it != m->eventQ.end(); it++) {
-        TimerEvent *temp = *it;
-        if (temp->id == id) {
-            m->eventQ.erase(it);
-            delete temp;
-            rc = 0;
-            break;
-        }
+    auto it = std::find_if(m->eventQ.begin(), m->eventQ.end(),
+                           [id](const auto& e) { return e.id == id; });
+    if (it != m->eventQ.end()) {
+        m->eventQ.erase(it);
+        return 0;
     }
-    return rc;
+
+    return -1;
 }
 
 int TimerThread::shutdown()
@@ -232,13 +212,7 @@ int TimerThread::shutdown()
     std::unique_lock<std::mutex> lck(m->mutex);
 
     m->inshutdown = 1;
-
-    /* Delete nodes in Q. Call registered free function on argument. */
-    for (auto & it : m->eventQ) {
-        delete it;
-    }
     m->eventQ.clear();
-
     m->condition.notify_all();
 
     while (m->inshutdown) {
