@@ -42,22 +42,18 @@
 #if EXCLUDE_WEB_SERVER == 0
 
 #include "webserver.h"
-
-#include <map>
-#include <iostream>
 #include <algorithm>
+#include <map>
+#include <unordered_map>
 #include <mutex>
-#include <condition_variable>
 
 #include <cinttypes>
 
-#include "httputils.h"
+#include "genut.h"
 #include "ssdplib.h"
 #include "statcodes.h"
-#include "upnp.h"
 #include "upnpapi.h"
-#include "VirtualDir.h"
-#include "genut.h"
+#include "uri.h"
 
 #include <cassert>
 #include <fcntl.h>
@@ -116,7 +112,7 @@ struct SendInstruction {
  * module variables - Globals, static and externs.
  */
 
-static const std::map<std::string, const char*> gEncodedMediaTypes = {
+static const std::unordered_map<std::string_view, std::string_view> gEncodedMediaTypes = {
     {"aif", "audio/aiff"},
     {"aifc", "audio/aiff"},
     {"aiff", "audio/aiff"},
@@ -191,14 +187,14 @@ static const std::map<std::string, const char*> gEncodedMediaTypes = {
 
 
 /*! Global variable. A file system directory which serves as webserver
-    root. If this is not set from the API UpnpSetWebServerRootDir()
-    call, we do not serve files from the file system at all (only
-    possibly the virtual dir and/or the localDocs). */
+  root. If this is not set from the API UpnpSetWebServerRootDir()
+  call, we do not serve files from the file system at all (only
+  possibly the virtual dir and/or the localDocs). */
 static std::string gDocumentRootDir;
 
 struct LocalDoc {
     std::string data;
-    time_t last_modified;
+    time_t last_modified{};
 };
 
 // Data which we serve directly: usually description
@@ -220,10 +216,9 @@ static std::mutex vdlmutex;
 
 
 /* Compute MIME type from file name extension. */
-static UPNP_INLINE int get_content_type(
-    const char *filename, std::string& content_type)
+static int get_content_type(const char* filename, std::string& content_type)
 {
-    const char *ctname = "application/octet-stream";
+    std::string_view ctname{"application/octet-stream"};
     content_type.clear();
     /* get ext */
     const char *e = strrchr(filename, '.');
@@ -239,23 +234,20 @@ static UPNP_INLINE int get_content_type(
     return 0;
 }
 
-int web_server_set_localdoc(
-    const std::string& path, const std::string& data, time_t last_modified)
+int web_server_set_localdoc(const std::string& path, const std::string& data, time_t last_modified)
 {
     if (path.empty() || path.front() != '/') {
         return UPNP_E_INVALID_PARAM;
     }
-    LocalDoc doc;
-    doc.data = data;
-    doc.last_modified = last_modified;
-    std::unique_lock<std::mutex> lck(gWebMutex);
+    LocalDoc doc{data, last_modified};
+    std::scoped_lock lck(gWebMutex);
     localDocs[path] = doc;
     return UPNP_E_SUCCESS;
 }
 
 int web_server_unset_localdoc(const std::string& path)
 {
-    std::unique_lock<std::mutex> lck(gWebMutex);
+    std::scoped_lock lck(gWebMutex);
     auto it = localDocs.find(path);
     if (it != localDocs.end())
         localDocs.erase(it);
@@ -284,10 +276,8 @@ static int get_file_info(const char *filename, struct File_Info *info)
     info->last_modified = s.st_mtime;
     int rc = get_content_type(filename, info->content_type);
     UpnpPrintf(UPNP_INFO, HTTP, __FILE__, __LINE__,
-               "get_file_info: %s, sz: %" PRIi64 ", mtime=%s rdable=%d\n",
-               filename, info->file_length,
-               make_date_string(info->last_modified).c_str(),
-               info->is_readable);
+               "get_file_info: %s, sz: %" PRIi64 ", mtime=%s rdable=%d\n", filename,
+               info->file_length, make_date_string(info->last_modified).c_str(), info->is_readable);
 
     return rc;
 }
@@ -311,8 +301,7 @@ int web_server_add_virtual_dir(
         return UPNP_E_INVALID_PARAM;
     }
 
-    UpnpPrintf(UPNP_DEBUG, HTTP, __FILE__, __LINE__,
-               "web_server_add_virtual_dir: [%s]\n", dirname);
+    UpnpPrintf(UPNP_DEBUG, HTTP, __FILE__, __LINE__, "web_server_add_virtual_dir: [%s]\n", dirname);
 
     VirtualDirListEntry entry;
     entry.cookie = cookie;
@@ -328,7 +317,7 @@ int web_server_add_virtual_dir(
         entry.path += '/';
     }
 
-    std::lock_guard<std::mutex> lock(vdlmutex);
+    std::scoped_lock lock(vdlmutex);
     auto old = std::find_if(virtualDirList.begin(), virtualDirList.end(),
                             [entry](const VirtualDirListEntry& o) {
                                 return entry.path == o.path;
@@ -339,7 +328,7 @@ int web_server_add_virtual_dir(
         }
         *old = entry;
     } else {
-        virtualDirList.push_back(entry);
+        virtualDirList.push_back(std::move(entry));
     }
     return UPNP_E_SUCCESS;
 }
@@ -349,7 +338,7 @@ int web_server_remove_virtual_dir(const char *dirname)
     if (dirname == nullptr) {
         return UPNP_E_INVALID_PARAM;
     }
-    std::lock_guard<std::mutex> lock(vdlmutex);
+    std::scoped_lock lock(vdlmutex);
     for (auto it = virtualDirList.begin(); it != virtualDirList.end(); it++) {
         if (it->path == dirname) {
             virtualDirList.erase(it);
@@ -362,7 +351,7 @@ int web_server_remove_virtual_dir(const char *dirname)
 
 void web_server_clear_virtual_dirs()
 {
-    std::lock_guard<std::mutex> lock(vdlmutex);
+    std::scoped_lock lock(vdlmutex);
     virtualDirList.clear();
 }
 
@@ -377,7 +366,7 @@ static const VirtualDirListEntry *isFileInVirtualDir(const std::string& path)
     // We ensure that vd entries paths end with /. Meaning that if
     // the paths compare equal up to the vd path len, the input
     // path is in a subdir of the vd path.
-    std::lock_guard<std::mutex> lock(vdlmutex);
+    std::scoped_lock lock(vdlmutex);
     for (const auto& vd : virtualDirList)
         if (!vd.path.compare(0, vd.path.size(), path, 0, vd.path.size()))
             return &vd;
@@ -385,49 +374,15 @@ static const VirtualDirListEntry *isFileInVirtualDir(const std::string& path)
     return nullptr;
 }
 
-/* Parse a Range header */
-static bool parseRanges(
-    const std::string& ranges, std::vector<std::pair<int64_t, int64_t>>& oranges)
-{
-    oranges.clear();
-    std::string::size_type pos = ranges.find("bytes=");
-    if (pos == std::string::npos) {
-        return false;
-    }
-    pos += 6;
-    bool done = false;
-    while(!done) {
-        std::string::size_type dash = ranges.find('-', pos);
-        if (dash == std::string::npos) {
-            return false;
-        }
-        std::string::size_type comma = ranges.find(',', pos);
-        std::string firstPart = ranges.substr(pos, dash-pos);
-        int64_t start = firstPart.empty() ? 0 : atoll(firstPart.c_str());
-        std::string secondPart = ranges.substr(
-            dash+1, comma != std::string::npos ?
-            comma-dash-1 : std::string::npos);
-        int64_t fin = secondPart.empty() ? -1 : atoll(secondPart.c_str());
-        std::pair<int64_t, int64_t> nrange(start,fin);
-        oranges.push_back(nrange);
-        if (comma != std::string::npos) {
-            pos = comma + 1;
-        }
-        done = comma == std::string::npos;
-    }
-    return true;
-}
-
 /*!
  * \brief Other header processing. Only HDR_ACCEPT_LANGUAGE for now.
  */
 static int CheckOtherHTTPHeaders(
-    MHDTransaction *mhdt, struct SendInstruction *RespInstr, int64_t)
+    const MHDTransaction* mhdt, struct SendInstruction* RespInstr, int64_t)
 {
-    for (const auto& header : mhdt->headers) {
+    for (const auto& [type, hvalue] : mhdt->headers) {
         /* find header type. */
-        int index = httpheader_str2int(header.first);
-        const std::string& hvalue = header.second;
+        int index = httpheader_str2int(type);
         if (index >= 0) {
             switch (index) {
             case HDR_ACCEPT_LANGUAGE:
@@ -474,8 +429,8 @@ static int process_request(
     std::vector<std::pair<int64_t, int64_t> > ranges;
     auto it = mhdt->headers.find("range");
     if (it != mhdt->headers.end()) {
-        if (parseRanges(it->second, ranges) && !ranges.empty()) {
-            if (ranges.size() > 1) {
+        if (parseHTTPRanges(it->second, ranges) && !ranges.empty()) {
+            if (ranges.size() > 1 || ranges[0].first == -1) {
                 return HTTP_REQUEST_RANGE_NOT_SATISFIABLE;
             }
 
@@ -514,7 +469,7 @@ static int process_request(
     }
     entryp = isFileInVirtualDir(request_doc);
     if (!entryp) {
-        std::unique_lock<std::mutex> lck(gWebMutex);
+        std::scoped_lock lck(gWebMutex);
         auto localdocit = localDocs.find(request_doc);
         // Just make a copy. Could do better using a
         // map<string,share_ptr> like the original, but I don't think
@@ -530,10 +485,8 @@ static int process_request(
         std::string qs;
         if (!mhdt->queryvalues.empty()) {
             qs = "?";
-            for (const auto& entry : mhdt->queryvalues) {
-                qs += query_encode(entry.first) + "=" + query_encode(entry.second);
-                qs += "&";
-            }
+            for (const auto& [name, val] : mhdt->queryvalues)
+                qs += query_encode(name) + "=" + query_encode(val) + "&";
             qs.pop_back();
         }
         std::string bfilename{filename};
@@ -774,13 +727,12 @@ static void web_server_callback(MHDTransaction *mhdt)
     }
 
     bool serverhfound{false};
-    for (const auto& header : headers) {
-        //std::cerr << "web_server_callback: adding header [" << header.first <<
-        //"]->[" << header.second << "]\n";
-        if (!stringlowercmp("server", header.first)) {
+    for (const auto& [name, val] : headers) {
+        // std::cerr << "web_server_callback: adding header [" << name << "]->[" << val << "]\n";
+        if (!stringlowercmp("server", name)) {
             serverhfound = true;
         }
-        MHD_add_response_header(mhdt->response, header.first.c_str(), header.second.c_str());
+        MHD_add_response_header(mhdt->response, name.c_str(), val.c_str());
     }
     if (!serverhfound) {
         MHD_add_response_header(mhdt->response, "SERVER", get_sdk_device_info("").c_str());
